import { useHttp } from '@crami/http';

export interface MetaData {
  connect: string;
  lang: string;
  companyId: string;
  userId: string;
}

export interface BoardMetaData extends MetaData {
  boardId: string,
}

export interface TreeMetaData extends MetaData {
  cardGroup: string,
}

export interface SimplifyCardData {
  // 虽然属性名为cardId，其实是指卡片编码属性
  cardId: string,
  cardName: string,
  abscissa: number | string,
  ordinate: number | string,
  width: number | string,
  height: number | string,
}

export interface CardData extends SimplifyCardData {
  boardId: string,
}

export interface AddPageMetaData extends MetaData {
  boardName: string,
  boardDefault: 'YSE' | 'NO',
  workBenchParas: SimplifyCardData[],
}

export interface EditPageMetaData extends AddPageMetaData {
  boardId: string,
  stateID: string,
  boardType: string,
  // stateID: 'ENABLE',
  // boardType: 'CUSTOM_CARD',
}

export interface BoardDetailMetaData {
  code: number,
  msg: string,
  data: {
    boardId: string,
    boardName: string,
    // 看板状态
    stateID: 'ENABLE',
    // 是否默认看板
    boardDefault: 'NO',
    // 看板类型
    boardType: 'CUSTOM_CARD',
    createUid: string,
    createTime: string,
    updateUid: string,
    updateTime: string,
    // 租户
    companyId: string,
    workBenchs: CardData[],
  }
}

// 新增、编辑看板
export async function saveBoard(params: AddPageMetaData | EditPageMetaData | any) {
  return useHttp({
    url: '/api/workbench/board/saveBoard',
    method: 'post',
    params,
  });
}

// 查询看板详情
export async function queryBoardDetail(params: BoardMetaData): Promise<BoardDetailMetaData> {
  return useHttp({
    url: '/api/workbench/board/queryBoard',
    method: 'post',
    params,
  });
}

// 停用看板
export async function disableBoard(params: BoardMetaData) {
  return useHttp({
    url: '/api/workbench/board/disableBoard',
    method: 'post',
    params,
  });
}

// 启用看板
export async function enableBoard(params: BoardMetaData) {
  return useHttp({
    url: '/api/workbench/board/enableBoard',
    method: 'post',
    params,
  });
}

// 查询卡片分类/卡片树接口
export async function getBoardCardTypeListTree(params: TreeMetaData) {
  return useHttp({
    url: '/api/workbench/cardType/getBoardCardTypeListTree',
    method: 'post',
    params,
  });
}

// 看板设为默认
export async function setDefaultBoard(params: BoardMetaData) {
  return useHttp({
    url: '/api/workbench/board/setDefaultBoard',
    method: 'post',
    params,
  });
}

// 查询我的看板列表
export async function getMyBoardList(params: MetaData) {
  return useHttp({
    url: '/api/workbench/board/getMyBoardList',
    method: 'post',
    params,
  });
}

