import request from '@/utils/request';
import { commonApi } from '@remote/icpx-platform/commonApi';

export interface MetaData {
  type: string;
  project: string;
  name: string;
}

export interface ProcessInfoData {
  processDefinitionName?: string;
  processState?: string;
  nodeTime?: string;
  remainderTime?: string;
  operationList?: string;
  routerName?: string;
  [key: string]: any;
}

export interface ApproverDataItem {
  key: string;
  [key: string]: any;
  approverType: string;
  approverName: string;
  approvalUserList: Array<any>;
  // users: UserItem[];
}
export interface MakerItemDataItem {
  key: string;
  [key: string]: any;
  ccType: string;
  ccName: string;
  makeUserList: Array<any>;
}
export interface ParameterDataItem {
  name: string;
  value: string;
}
export interface ProcessDataBase {
  processDefinitionName: string;
  processDefinitionKey: string;
  categoryId: string;
  processDesc: string;
  launchDisplay: string;
}
export interface ProcessCopyListMember {
  makeType?: string; // "user",
  makeTypeName?: string; // "用户"
  optionList?: Array<any> | string;
  key?: string;
  [key: string]: any;
}
export interface ProcessDataCopyList {
  makeTimeType?: string; // "抄送时间类型"，   1、流程结束(默认)。0、流程发起
  makeList?: Array<ProcessCopyListMember>;
}
// 发起人范围信息
export interface ProcessDataSpon {
  sponType: string;
  sponTypeName: string;
  sponUsers: Array<any> | string;
  key: string;
  [key: string]: any;
}
export interface ProcessDataFormInfo {
  bindFormType?: string; // '绑定表单类型';
  defaultFormId?: string; // '在线表单Id';
  pageCode?: string; // '表单编码';
  defaultRouterName?: string; // '在线表单的缺省路由名称';
}
export interface ProcessNodeModel {
  nodeKey?: string; //"节点编码",
  nodeName?: string; //"节点名称",
  allowUpdate?: string; //"允许修改",  1允许/0禁止。
  acceptanceEmpty?: string; //"验收手动指定为空"   1允许/0不允许
}
export interface PlugItem {
  id: string;
  name: string;
  page: string;
}
export interface ProcessData extends ProcessDataBase, ProcessDataFormInfo {
  sponScope?: string; //发起人范围 1无限制/0限制范围
  flowSponDtoList?: Array<ProcessDataSpon>;
  // [{
  //     sponType:string;
  //     sponTypeName:string;
  //     "sponUsers": [{
  //       "sponId":string;
  //       "sponName":string;
  //     }]
  //   }],
  modelScope?: string; //"适用模型是否限制",  1无限制/0适用模型限制范围
  flowApplyModelDtoList?: Array<any>;
  // [{
  //   "processApplyModelId"?: string;//"流程适用模型对象id",
  //   "processApplyModelName"?: string;//"流程适用模型对象名称"
  // }];
  sponRetreat?: string; //"发起人是否可撤回",   1允许/0禁止
  flowNodes?: Array<string>; // ['', ''];
  updateApprover?: string; //"能否修改执行人",     1允许/0禁止
  flowNodeModel?: {
    modifier_type?: Array<string>; //["sponsor","executor"],
    flowNodeModels?: Array<ProcessNodeModel>;
  };
  canUrge?: string; //"能否催办",   1允许/0禁止
  flowMakeUp?: ProcessDataCopyList; //抄送人
  pluginData?: Array<PlugItem>;
  bpmnXml?: string; // '流程定义的xml';
}
export async function getByProjectAndType(data: MetaData) {
  return request.request<MetaData, any>({
    url: '/studio-cache/page/getMetaInfo',
    method: 'POST',
    data,
  });
}
export async function getFlowCategory() {
  return request.request<MetaData, any>({
    url: '/system/model/getSltData',
    method: commonApi().system.getSltData.method,
    data: {
      ...commonApi().system.getSltData.params,
      paraList: '{}',
      type: 'tree',
      // userId: 'P_SYS_USER_0162a3c2c85c45f2b4fa3baffc350d6d',
      viewName: 'P_FLOW_CLASSIFYPULL_LISTVIEW',
    },
  });
}
