import { useHttp } from '@crami/http';
import { useI18n } from '@crami/locale';
import { getInfo, ICP_CONFIG } from '@remote/icpx-platform/utils';
import type { BaseApi } from '@crami/http';
import { omit, pick } from 'lodash';
export interface DynamicLocaleParams {
  project?: string;
  lang?: string[];
}
export interface ICustomLocale {
  // 如果要返回全部字段，则返回空数组
  navs?: Array<{ [key: string]: Array<any> }>;
  views?: Array<{ [key: string]: Array<any> }>;
  menus?: Array<{ [key: string]: Array<any> }>;
  customs?: Array<{ [key: string]: Array<any> }>;
}
const LOCALE_API = '/api/studio-cache/dic/getGlobalDic';
export const getDynamicLocale = async (params?: DynamicLocaleParams, config?: Record<any, any>) => {
  const i18n = useI18n();
  const userInfo = getInfo(config ? config : ICP_CONFIG);
  const apiInfo: BaseApi = {
    url: LOCALE_API,
    method: 'post',
    params: {
      project: params?.project ? params.project : userInfo.projectCode || '',
      lang: params?.lang ? params.lang : ['zhs', 'en'],
    },
  };
  const result = await useHttp(apiInfo);
  if (result.code === 200) {
    const data: Record<any, any> = result.data || {};
    // 将获取到的数据合并入多语言
    i18n.mergeLocaleMessage('zh-CN', {
      antd: {
        ...omit(data.zhs || {}, ['navs']),
      },
      ...pick(data.zhs || {}, ['navs']),
    });
    i18n.mergeLocaleMessage('en-US', {
      antd: {
        ...omit(data.en || {}, ['navs']),
      },
      ...pick(data.en || {}, ['navs']),
    });
  }
};
// 获取执行多语言
export const getCustomLocale = async (params: ICustomLocale, config?: any) => {
  const i18n = useI18n();
  const userInfo = getInfo(config ? config : ICP_CONFIG);

  const result = await useHttp({
    url: '/api/studio-cache/dic/getGeneralDic',
    method: 'post',
    params: {
      project: userInfo.project || '',
      lang: ['zhs', 'en'],
      params: {
        ...params,
      },
    },
  });

  // 合并多语言
  if (result.code === 200) {
    const data = result.data || {};
    // 将获取到的数据合并入多语言
    for (const key in data) {
      if (Object.prototype.hasOwnProperty.call(data, key)) {
        switch (key) {
          // 中文
          case 'zhs':
            i18n.mergeLocaleMessage('zh-CN', data[key]);
            break;
          // 英文
          case 'en':
            i18n.mergeLocaleMessage('en-US', data[key]);
            break;
          default:
            i18n.mergeLocaleMessage(key as string, data[key]);
            break;
        }
      }
    }
  }
};

// demo
// const localeParams = {
//   views: [
//     {
//       P_MSG_METADATA_TEMPLETAView: ['EID', 'ENAME'],
//     },
//     {
//       P_STO_STORAGE_VIEW: [],
//     },
//   ],
//   navs: [
//     {
//       NavigationItems: ['P_STO_APPLICATION', 'P_STO_DUPLICATION_STRATEGY_PAGE', 'P_STO_FILE_NAV'],
//     },
//   ],
//   menus: [
//     {
//       P_MSG_CHANNELMenu: ['enable', 'disable', 'deleteRow'],
//     },
//   ],
//   customs: [
//     {
//       Common: [],
//     },
//   ],
// };
// getCustomLocale(localeParams);
// t('views.P_STO_STORAGE_VIEW.STORAGE_DESCRIPTION');
