import type { RouteRecordRaw } from 'vue-router';
import type { MenuDataItem } from '@/router/typing';
import type { RouteProps } from '@/components/base-layouts/typing';
import RouteView from '@/layouts/route-view.vue';
import { h, defineAsyncComponent } from 'vue';
import localeValues from 'ant-design-vue/es/locale/zh_CN';

export const filterMenu = (routes: MenuDataItem[]): RouteRecordRaw[] => {
  return routes.find(item => item.name === 'index')?.children || [];
};

const staticRoutes: Record<string, any> = {};

export function loadStaticRoutes(routers: RouteRecordRaw[] = []) {
  routers.forEach(route => {
    staticRoutes[route.path] = route;
  });
}

export function getRouteComponent(filePath = '') {
  return staticRoutes[filePath.toLowerCase()]?.component;
}

export const changeMenu = (originRoutes: any) => {
  if (Array.isArray(originRoutes)) {
    const routes: MenuDataItem[] = [];
    for (let i = 0; i < originRoutes.length; i++) {
      const item = originRoutes[i];
      const { children: _, ...iItem } = item;
      if (item.path == null) {
        continue;
      }

      const fItem: MenuDataItem = {
        name: item.name,
        path: item.path,
        redirect: item.redirect,
        meta: {
          icon: item.iconName || '',
          // title: item.text || '',
          title: item.locale || '',
          keepAlive: item.keepalive,
          routerId: item.id,
          // 增加多语言引用
          locale: item.locale,
        },
        extra: { ...iItem },
        component: item.redirect
          ? undefined
          : getRouteComponent(item.filePath)
          ? getRouteComponent(item.filePath)
          : getRouteComponent('/platform/NotFount/Index'),

        //   h(RouteView, null, () =>
        //   h(defineAsyncComponent(() => import(/* webpackChunkName: "dashboard" */ '@/views/home.vue'))),
        // ),
        // item.parentId == undefined
        //   ? item.children && item.children.length > 0
        //     ? RouteView
        //     : // : h(RouteView, null, () => h(getRouteComponent(item.path)))
        //       h(RouteView, null, () =>
        //         h(
        //           defineAsyncComponent(
        //             getRouteComponent(item.path)
        //               ? getRouteComponent(item.path)
        //               : getRouteComponent('/platform/NotFount/Index'),
        //           ),
        //         ),
        //       )
        //   : getRouteComponent(item.filePath)
        //   ? getRouteComponent(item.filePath)
        //   : getRouteComponent('/platform/NotFount/Index'),
      };
      //针对于隐藏子路由的逻辑处理
      fItem.meta.hideChildrenInMenu = item['hideChildren'] || false;
      fItem.meta.hideInMenu = item['IsHidden'] || false;
      fItem.meta.frontHidden = false;
      if (item.children && item.children.length > 0) {
        fItem['children'] = changeMenu(item.children);
      }
      routes.push(fItem);
    }
    return routes;
  } else {
    const routes: MenuDataItem[] = [];
    const { children: _, ...iItem } = originRoutes;
    const fItem: MenuDataItem = {
      name: originRoutes.name,
      path: originRoutes.path,
      redirect: originRoutes.redirect,
      meta: {
        icon: originRoutes.iconName || '',
        title: originRoutes.text,
        keepAlive: originRoutes.keepalive,
      },
      extra: { ...iItem },
      component: originRoutes.redirect
        ? undefined
        : getRouteComponent(originRoutes.filePath)
        ? getRouteComponent(originRoutes.filePath)
        : getRouteComponent('/platform/NotFount/Index'),
      // originRoutes.parentId == undefined
      //   ? originRoutes.children && originRoutes.children.length > 0
      //     ? RouteView
      //     : h(RouteView, null, () =>
      //         h(
      //           defineAsyncComponent(
      //             getRouteComponent(originRoutes.path)
      //               ? getRouteComponent(originRoutes.path)
      //               : getRouteComponent('/platform/NotFount/Index'),
      //           ),
      //         ),
      //       )
      //   : getRouteComponent(originRoutes.filePath)
      //   ? getRouteComponent(originRoutes.filePath)
      //   : getRouteComponent('/platform/NotFount/Index'),
    };
    //针对于隐藏子路由的逻辑处理
    fItem.meta.hideChildrenInMenu = originRoutes['hideChildren'] || false;
    fItem.meta.hideInMenu = originRoutes['IsHidden'] || false;
    fItem.meta.frontHidden = false;
    if (originRoutes.children && originRoutes.children.length > 0) {
      fItem['children'] = changeMenu(originRoutes.children);
    }
    routes.push(fItem);
    return routes;
  }
};

interface MenuMap {
  parentKeys?: string[];
}
type MenuKeyMap = Record<string, MenuMap>;
interface MenuInfo {
  menus: RouteProps[];
  menuKeyMap: MenuKeyMap;
}
export const genMenuInfo = (routes: RouteRecordRaw[]): MenuInfo => {
  const menuKeyMap: MenuKeyMap = {};
  const getMenuInfo = (routes: RouteRecordRaw[], parentKeys: string[] = []): RouteProps[] => {
    return routes.map(route => {
      const currentRouter: RouteProps = {
        ...route,
      };
      menuKeyMap[currentRouter.path] = { parentKeys: parentKeys };
      if (route.meta?.hideChildrenInMenu) {
        route.children &&
          route.children
            .map(item => item.path)
            .forEach(
              value => (menuKeyMap[value] = { parentKeys: [...parentKeys, currentRouter.path] }),
            );
      }
      // 是否有子菜单，并递归处理
      if (route.children && route.children.length > 0 && !route.meta?.hideChildrenInMenu) {
        // Recursion
        currentRouter.children = getMenuInfo(route.children, [...parentKeys, currentRouter.path]);
      } else {
        currentRouter.children = undefined;
      }

      return currentRouter;
    });
  };
  const menus = getMenuInfo(routes);
  return {
    menus,
    menuKeyMap,
  };
};

export const getMenuFirstChildren = (menus: RouteProps[]) => {
  return menus.map(menu => {
    // eslint-disable-next-line
    const { children: _children, ...rest } = menu;
    return rest;
  });
};

export const findMenuChildren = (
  menus: RouteProps[] | undefined,
  key: string | symbol | undefined,
) => {
  return (menus && menus.find(item => item.name === key)?.children) || [];
};

export const getActiveId = (route: string[], path: string) => {
  for (let i = 0; i < route.length; i++) {
    if (route[i]?.path == path) {
      return route[i];
    }
    if (route[i]?.children && route[i]?.children.length) {
      const val = getActiveId(route[i].children, path);
      if (val) return val;
    }
  }
};
