import type { PropType } from 'vue';
import { defineComponent } from 'vue';
import './style.less';
// import { DIGIT_MIN_VALUE, DIGIT_MAX_VALUE } from './ValueComponent';
const radioBoolData = [
  { label: 'true', value: 'true' },
  { label: 'false', value: 'false' },
];

const valueProps = {
  value: {
    type: [Array, Object, String, Number] as PropType<string | number | any>,
    default: undefined,
  },
  valueType: {
    type: String as PropType<string>,
    default: '02',
  },
  min: {
    type: [String, Number] as PropType<string | number>,
    default: undefined,
  },
  //  最大值
  max: {
    type: [String, Number] as PropType<string | number>,
    default: undefined,
  },
  options: {
    type: Array as PropType<Array<any>>,
    default: undefined,
  },
};

export default defineComponent({
  name: 'EditValueComponent',
  emits: ['change', 'click'],
  props: valueProps,
  setup(props, { attrs, emit, slots, expose }) {
    const handleClick = e => {
      emit('click', e, props.valueType);
    };
    // const formatterNumber = value => `${value}`.replace('.', '');
    return () => {
      let dom = null;
      const { value, valueType: type, max, min, options } = props;
      // console.log('value component value:' + value);
      // console.log('value component type:' + type);
      if (type === '02') {
        //字符串
        dom = <h-input value={value} maxlength={4000} {...attrs}></h-input>;
      } else if (type === '01') {
        //  布尔
        dom = <h-radio-group options={radioBoolData} value={value} {...attrs}></h-radio-group>;
      } else if (type === '03') {
        //整数
        dom = (
          <h-input-number
            step="1"
            precision="0"
            formatter={value => `${value}`.replace('.', '')}
            value={value}
            max={max ? max : Number.MAX_SAFE_INTEGER}
            min={min ? min : Number.MIN_SAFE_INTEGER}
            {...attrs}
          ></h-input-number>
        );
      } else if (type === '04') {
        // 小数  stringMode={true}
        dom = (
          <h-input-number
            value={value}
            stringMode
            max={max ? max : Number.MAX_SAFE_INTEGER}
            min={min ? min : Number.MIN_SAFE_INTEGER}
            {...attrs}
          ></h-input-number>
        );
      } else if (type === '05') {
        // 下拉框-单选
        dom = (
          <h-select
            not-found-content={null}
            optionLabelProp={null}
            value={value}
            options={options}
            {...attrs}
          ></h-select>
        );
      } else if (type === '06') {
        // 下拉框-多选
        dom = (
          <h-select
            mode="multiple"
            not-found-content={null}
            optionLabelProp={null}
            value={value}
            options={options}
            {...attrs}
          ></h-select>
        );
      } else if (type === '07') {
        // 自定义
        dom = (
          <div style={{ display: 'flex', alignItems: 'center' }}>
            <h-input value={value} disabled {...attrs}></h-input>
            <div class="select_value_component">
              <h-icon
                style={{ padding: '8px' }}
                name="hoteam-setting"
                size="14"
                onClick={handleClick}
              ></h-icon>
            </div>
          </div>
        );
      }
      return dom;
    };
  },
});
