import type { PropType } from 'vue';
import { reactive, defineComponent } from 'vue';
import './style.less';
const radioBoolData = [
  { label: 'true', value: 'true' },
  { label: 'false', value: 'false' },
];

const valueProps = {
  value: {
    type: [Array, Object, String, Number] as PropType<string | number | any>,
    default: undefined,
  },
  valueType: {
    type: String as PropType<string>,
    default: '02',
  },
  min: {
    type: [String, Number] as PropType<string | number>,
    default: undefined,
  },
  //  最大值
  max: {
    type: [String, Number] as PropType<string | number>,
    default: undefined,
  },
  options: {
    type: Array as PropType<Array<any>>,
    default: undefined,
  },
};

export default defineComponent({
  name: 'ReadonlyValueComponent',
  props: valueProps,
  setup(props) {
    // const formatterNumber = value => `${value}`.replace('.', '');
    return () => {
      let dom = null;
      const { value, valueType: type, options } = props;

      const handleOptionsToObject = (options: Array<any>) => {
        const tempData = {};
        options.map(item => {
          tempData[item.value] = item.label;
        });
        return tempData;
      };
      const getSelectLabel = (options: Record<string, unknown>, selectValue: Array<string>) => {
        let val = '';
        selectValue.map((item, index) => {
          if (index > 0) {
            val = val + `,${options[item]}`;
          } else {
            val = val + `${options[item]}`;
          }
        });
        return val;
      };
      const selectValueDom = (options: Record<string, unknown>, selectValue: Array<string>) => {
        return selectValue.map((item, index) => {
          if (index > 0) {
            return <span> ,{options[item]}</span>;
          } else {
            return <span> {options[item]}</span>;
          }
        });
      };

      if (type === '02' || type === '01' || type === '03' || type === '04' || type === '07') {
        //字符串
        dom = (
          <span title={value} style="white-space: pre;">
            {value}
          </span>
        );
      } else if (type === '05') {
        // 下拉框-单选
        const data = handleOptionsToObject(options)[value];
        dom = (
          <span title={data} style="white-space: pre;">
            {data}
          </span>
        );
      } else if (type === '06') {
        // 下拉框-多选
        const data = getSelectLabel(handleOptionsToObject(options), value.split(','));
        dom = (
          <span title={data} style="white-space: pre;">
            {data}
          </span>
        );
      }
      return dom;
    };
  },
});
