import { mapping as customMapping } from './custom';
import { mapping as systemMapping } from './system';

const mapping = Object.assign({}, customMapping, systemMapping);

const getFile = path => {
  return new Promise((resolve, reject) => {
    import(/* @vite-ignore */ path)
      .then(card => {
        resolve(card.default);
      })
      .catch(err => {
        reject(err);
      });
  });
};

/**
 * 根据路径映射获取卡片文件
 * @param {string} cardId 卡片编码
 * @returns {Promise}
 */
const getCardByMapping = cardId => {
  const path = mapping[cardId];
  if (!path) return Promise.reject('未找到卡片映射');
  return getFile(`./../cardComponents${path}`);
};

/**
 * 根据名称获取卡片文件
 * @param {string} name 卡片编码/文件名称
 * @returns {Promise}
 */
const getCardByFileName = name => {
  const files = import.meta.glob('./../cardComponents/**/**.{vue,js,jsx,ts,tsx}');
  let targetPath = '';
  const fileList = Object.keys(files);

  try {
    for (const path of fileList) {
      const category = path.split('/');
      const fileName = category.pop().replace(/.(vue|js|jsx|ts|tsx)/, '');
      if (fileName === name || (['Index', 'index'].includes(fileName) && category.pop() === name)) {
        targetPath = path;
        break;
      }
    }
  } catch (err) {
    return Promise.reject(err);
  }

  return targetPath ? getFile(targetPath) : Promise.reject('卡片不存在');
};

/**
 *
 * @param {String} cardId  卡片编码
 * @param {String} mode name|mapping
 * @param {Object} options 其它
 * @returns {Promise}
 */
export const getCardComponent = (cardId, mode = 'mapping') => {
  switch (mode) {
    case 'mapping':
      return getCardByMapping(cardId);
    case 'name':
      return getCardByFileName(cardId);
    default:
      return Promise.reject('mode参数错误');
  }
};
