import { GridStack } from 'gridstack';
import { cloneDeep } from 'lodash-es';

(function () {
  // 是否碰撞计算
  let isIntercepted = function (a, b) {
    return !(a.y >= b.y + b.h || a.y + a.h <= b.y || a.x + a.w <= b.x || a.x >= b.x + b.w);
  }

  // 确定是否碰撞
  let collide = (nodes, skip, area = skip) => {
    return nodes.find(node => node !== skip && isIntercepted(node, area));
  }

  GridStack.prototype._load = function (nodes) {
    this._nodes = cloneDeep(nodes);
    return this
  }

  /**
   * 看板卡片水平方向左挤压功能
   */
  GridStack.prototype._packHorizontalNodes = function (nodes, allNodes) {
    let opts = this.opts || {};
    let row = opts.maxRow || opts.minRow || opts.row || 100;

    if (!allNodes) {
      allNodes = this._getNodes();
      !allNodes.length && (allNodes = this._getPreNodes());
    }
    !nodes && (nodes = allNodes);
    !Array.isArray(nodes) && (nodes = [nodes]);

    if (allNodes.length === 1) {
      nodes[0].x = 0;
      return this;
    }

    // 确保x轴的计算权重始终大于y轴
    nodes.sort((a, b) => (a.x * row + a.y) - (b.x * row + b.y));

    nodes.forEach((node) => {
      while (node.x > 0) {
        let newX = node.x - 1;
        let canBeMoveLeft = !collide(allNodes, node, { x: newX, y: node.y, w: node.w, h: node.h });
        if (!canBeMoveLeft) break;
        node.x = newX;
      }
    });

    return this;
  }

  /**
   * 看板若存在整行空白则卡片上移填充空白
   */
  GridStack.prototype._clearInvalidRow = function (nodes) {
    // 默认向上浮动，则按默认规则来
    let opts = this.opts;
    if (!(opts && opts.float)) return this;

    !nodes && (nodes = this._getNodes());

    nodes.sort((a, b) => a.y - b.y);

    // 计算y轴占用空间
    let section = [{ start: 0, end: 0, gap: 0 }];
    nodes.forEach(node => {
      let region = section[section.length - 1];
      (node.y >= region.start && node.y <= region.end) ?
        region.end = Math.max(region.end, node.y + node.h) :
        section.push({
          start: node.y,
          end: node.y + node.h,
          gap: node.y - region.end,
        });
    })

    // 卡片上移
    let index = 0;
    let gapSum = 0;
    nodes.forEach(node => {
      let region = section[index];
      if (node.y >= region.end) {
        index++;
        gapSum = gapSum += section[index].gap;
      }
      (gapSum > 0) && (node.y -= gapSum);
    });

    return this;
  }

  /**
   * 设置看板行数值
   * @param {number} row 
   * @returns gridstack
   */
  GridStack.prototype._setRow = function (row = 12) {
    this.opts.maxRow = row;
    this.opts.minRow = row;
    return this;
  }

  GridStack.prototype._getNodes = function () {
    return this.engine.nodes;
  }

  GridStack.prototype._getPreNodes = function () {
    return this._nodes || [];
  }

  /**
   * 根据节点的属性坐标重置节点对应的dom位置
   * @param {array, object} nodes 
   */
  GridStack.prototype._resetNodesRealPosi = function (nodes) {
    let resetPosi = (node) => {
      if (!node.el) return;
      this._writePosAttr(node.el, node);
    }

    if (!nodes) {
      nodes = this._getNodes();
    }

    Array.isArray(nodes) ?
      nodes.forEach(node => resetPosi(node)) :
      resetPosi(nodes);

    return this;
  }

  let _leave = GridStack.prototype._leave;
  GridStack.prototype._leave = function (el, helper) {
    _leave.call(this, el, helper);
  }

  GridStack.prototype._insideBoard = function (el) {
    if (!el) return;
    return this.el.contains(el);
  }
})();