import { throttle } from 'lodash-es';

const dynamicHeight = {
  mounted(el) {
    const handleResize = throttle(() => {
      let screenHeight = document.documentElement.clientHeight;
      let style = el.getBoundingClientRect();
      let y = style.y;
      let height = screenHeight - y - 24;
      if (height < 360) {
        height = 360;
      }
      el.style.height = `${height}px`;
    }, 50);

    el._handleResize = handleResize;
    window.addEventListener('resize', el._handleResize);

    handleResize();
  },

  beforeUnmount(el) {
    window.removeEventListener('resize', el._handleResize);
    el._handleResize = null;
  },
};

export default dynamicHeight;
