import { WIDGET_CARD_CLS, WIDGET_CARD_CLS, DRAG_HANDLE_CLS } from '@/components/widget-board/const';

// 为什么要通过此步动态配置支持自定义拖拽的node节点(gridstack v5.1.1)：
// gridstack.js 通过配置option.handle 和 options.handleClass 属性来支持自定义的拖拽node，不配置的话默认整个widgetItem都可拖拽移动
// gridstack.js 支持拖拽功能是在初始化执行时内部执行addWidget()后 使用querySelector查找当前widgetItem下符合自定义配置handle\handleClass属性的node，如果没找到则将当前的widgetItem根dom作为可支持拖拽的node，且整个过程都为同步执行，而不是使用事件委托的方式支持动态创建widgetItem配置自定义拖拽node
// vue.js 的node大体为异步渲染，且该看板功能会存在需要异步执行的操作，导致gridstack.js的自定义node支持拖拽的配置失效
// 确保gridstack.js的同步操作功能正常执行，需要同步渲染的方式创建don。所以使用vue.js 的mount() 而不是Teleport等其它方式创建card内容
// 为了支持异步渲染的card内容里存在gridstack同步执行配置好的可拖拽自定义node，现在采取的方式是在widgetItem创建时创建一个用于支持自定义拖拽的node，在异步创建的card渲染完后将该自定义node移动到card的node节点下面
// vue3 vnode在正常渲染下使用insertBefore 可能会报错，使用createBlock创建相应节点
// girdstack为node配置拖拽事件执行时间早于该指令hooks执行时间

const normalizeDragNodeStyle = (el) => {
  el.style.position = 'absolute';
  el.style.left = '0';
  el.style.top = '0';
  el.style.right = '0';
  el.style.bottom = '0';
}

const enableDrag = {
  // 将用于支持自定义拖拽的widget node节点移动到异步创建好的卡片node下面
  mounted(el, binding, vnode) {
    const PROCESS = 'PROCESS';
    const SUCCESS = 'SUCCESS';
    const FAILED = 'FAILED';
    let status = PROCESS;
    let currentNode = el;
    let nodeLevel = 0;
    let dragNode;

    while (status === PROCESS) {
      // 自定义卡片容器node
      if (currentNode.classList.contains(WIDGET_CARD_CLS)) {
        dragNode = currentNode.querySelector(`.${DRAG_HANDLE_CLS}`);
        if (dragNode) {
          const computedStyle = getComputedStyle(el);
          if (computedStyle.position === 'static') {
            // 使用insertBefore而不是appendChild消除对原有node的样式影响
            el.insertBefore(currentNode.removeChild(dragNode), el.childNodes[0]);
            el.style.position = 'relative';
            dragNode.style.cursor = 'move';
            status = SUCCESS;
          } else {
            status = FAILED;
          }
        }
        else {
          status = FAILED;
        }
      }
      
      if (++nodeLevel > 10) {
        status = FAILED;
      }
      
      currentNode = currentNode.parentNode;
    }
    
    if (status === SUCCESS) {
      normalizeDragNodeStyle(dragNode);
    } else if (dragNode) {
      // 标记的node拖拽失败时会造成整个widget也无法拖拽
      dragNode.parentNode.removeChild(dragNode);
    } else {
      // 默认整个widget都可拖拽
    }
  },

  // 将用于拖拽的node节点从card node还原到原来的地方，因为可能会刷新卡片内容，此时原来卡片可拖拽的节点不见了，会造成刷新后的card不能拖拽
  beforeUnmount(el) {
    const PROCESS = 'PROCESS';
    const SUCCESS = 'SUCCESS';
    let status = PROCESS;
    let currentNode = el;
    let nodeLevel = 0;

    while (status === PROCESS) {
      if (currentNode.classList.contains(WIDGET_CARD_CLS)) {
        let dragNode = currentNode.querySelector(`.${DRAG_HANDLE_CLS}`);
        if (dragNode) {
          // 使用insertBefore而不是appendChild消除对原有node的样式影响
          currentNode.insertBefore(dragNode.parentNode.removeChild(dragNode), currentNode.childNodes[0]);
          dragNode.style.cursor = 'default';
          status = SUCCESS;
        }
        // 未配置拖拽节点则使用默认设置整个卡片都可拖拽 
        else {
          status = SUCCESS;
        }
      }

      if (++nodeLevel > 10) {
        status = SUCCESS;
      }

      currentNode = currentNode.parentNode;
    } 
  }
}

export default enableDrag;