import { ref, onMounted, onUnmounted } from 'vue';
import { throttle } from 'lodash-es';
import { addListener, removeListener } from 'resize-detector';
import { WORKBENCH_CARD_CLS } from '@/components/widget-board/const';

const useHandleResize = (bodyRef) => {
  const bodySizeCls = ref('');

  onMounted(() => {
    const resize = throttle((e) => {
      let height = parseInt(window.getComputedStyle(e).height.replace('px', ''));
      if (height && height < 108) {
        bodySizeCls.value = `${WORKBENCH_CARD_CLS}-small-body`;
      } else {
        bodySizeCls.value = ``;
      }
    }, 300)
    bodyRef.value._resizeHandler = resize;
    addListener(bodyRef.value, bodyRef.value._resizeHandler);
  })

  onUnmounted(() => {
    // keep-alive失效时触发
    if (!bodyRef.value) return;
    removeListener(bodyRef.value, bodyRef.value._resizeHandler);
    bodyRef.value._resizeHandler = null;
  })

  return {
    bodySizeCls,
  }
}

export default useHandleResize;