import { assign } from 'min-dash';

/**
 * A palette provider for BPMN 2.0 elements.
 * 自定义左侧工具栏各个工具初始化的名称、图标、class
 */
export default function PaletteProvider(
  palette,
  create,
  elementFactory,
  spaceTool,
  lassoTool,
  handTool,
  globalConnect,
  translate,
) {
  this._palette = palette;
  this._create = create;
  this._elementFactory = elementFactory;
  this._spaceTool = spaceTool;
  this._lassoTool = lassoTool;
  this._handTool = handTool;
  this._globalConnect = globalConnect;
  this._translate = translate;

  palette.registerProvider(this);
}

PaletteProvider.$inject = [
  'palette',
  'create',
  'elementFactory',
  'spaceTool',
  'lassoTool',
  'handTool',
  'globalConnect',
  'translate',
];

PaletteProvider.prototype.getPaletteEntries = function () {
  var actions = {},
    create = this._create,
    elementFactory = this._elementFactory,
    spaceTool = this._spaceTool,
    lassoTool = this._lassoTool,
    handTool = this._handTool,
    globalConnect = this._globalConnect,
    translate = this._translate;

  function createAction(type, group, className, title, options) {
    function createListener(event) {
      var shape = elementFactory.createShape(assign({ type: type }, options));
      var arr = shape.id.split('_');

      if ('bpmn:StartEvent' == type) {
        shape.businessObject.name = `开始`;
      } else if ('bpmn:EndEvent' == type) {
        shape.businessObject.name = `结束`;
      } else if ('bpmn:ParallelGateway' == type ||
      'bpmn:ExclusiveGateway' == type ||
      'bpmn:InclusiveGateway' == type
    ) {
      shape.businessObject.name =
        arr?.length > 1 ? `网关_${arr[1]}` : `网关_${new Date().getTime()}`;
    } else {
        if (arr.length > 1) {
          shape.businessObject.name = `节点_${arr[1]}`;
        } else {
          shape.businessObject.name = `节点_${new Date().getTime()}`;
        }
      }

      if (options) {
        shape.businessObject.di.isExpanded = options.isExpanded;
      }

      create.start(event, shape);
    }

    var shortType = type.replace(/^bpmn:/, '');

    return {
      group: group,
      className: className,
      title: title || translate('Create {type}', { type: shortType }),
      action: {
        dragstart: createListener,
        click: createListener,
      },
    };
  }

  // function createSubprocess(event) {
  //   var subProcess = elementFactory.createShape({
  //     type: 'bpmn:SubProcess',
  //     x: 0,
  //     y: 0,
  //     isExpanded: true,
  //   });
  //   //节点创建名称
  //   var arr = subProcess.id.split('_');
  //   if (arr.length > 1) {
  //     subProcess.businessObject.name = `节点_${arr[1]}`;
  //   } else {
  //     subProcess.businessObject.name = `节点_${new Date().getTime()}`;
  //   }

  //   var startEvent = elementFactory.createShape({
  //     type: 'bpmn:StartEvent',
  //     x: 40,
  //     y: 82,
  //     parent: subProcess,
  //   });
  //   //节点创建名称
  //   var arr2 = startEvent.id.split('_');
  //   if (arr2.length > 1) {
  //     startEvent.businessObject.name = `节点_${arr2[1]}`;
  //   } else {
  //     startEvent.businessObject.name = `节点_${new Date().getTime()}`;
  //   }

  //   create.start(event, [subProcess, startEvent], {
  //     hints: {
  //       autoSelect: [startEvent],
  //     },
  //   });
  // }

  // function createParticipant(event) {
  //   var shape = elementFactory.createParticipantShape();
  //   //节点创建名称
  //   var arr = shape.id.split('_');
  //   if (arr.length > 1) {
  //     shape.businessObject.name = `节点_${arr[1]}`;
  //   } else {
  //     shape.businessObject.name = `节点_${new Date().getTime()}`;
  //   }
  //   create.start(event, shape);
  // }

  assign(actions, {
    'hand-tool': {
      group: 'tools',
      className: 'bpmn-icon-hand-tool',
      title: translate('Activate the hand tool'),
      action: {
        click: function (event) {
          handTool.activateHand(event);
        },
      },
    },
    'lasso-tool': {
      group: 'tools',
      className: 'bpmn-icon-lasso-tool',
      title: translate('Activate the lasso tool'),
      action: {
        click: function (event) {
          lassoTool.activateSelection(event);
        },
      },
    },
    'space-tool': {
      group: 'tools',
      className: 'bpmn-icon-space-tool',
      title: translate('Activate the create/remove space tool'),
      action: {
        click: function (event) {
          spaceTool.activateSelection(event);
        },
      },
    },
    'global-connect-tool': {
      group: 'tools',
      className: 'bpmn-icon-connection-multi',
      title: translate('Activate the global connect tool'),
      action: {
        click: function (event) {
          globalConnect.toggle(event);
        },
      },
    },
    'tool-separator': {
      group: 'tools',
      separator: true,
    },
    'create.start-event': createAction(
      'bpmn:StartEvent',
      'event',
      'bpmn-icon-start-event-none',
      translate('Create StartEvent'),
    ),
    // 'create.intermediate-event': createAction(
    //   'bpmn:IntermediateThrowEvent',
    //   'event',
    //   'bpmn-icon-intermediate-event-none',
    //   translate('Create Intermediate/Boundary Event'),
    // ),
    'create.end-event': createAction(
      'bpmn:EndEvent',
      'event',
      'bpmn-icon-end-event-none',
      translate('Create EndEvent'),
    ),
    'create.user-task': createAction(
      'bpmn:UserTask',
      'activity',
      'bpmn-icon-user-task',
      translate('Create User Task'),
    ),
    'create.exclusive-gateway': createAction(
      'bpmn:ExclusiveGateway',
      'gateway',
      'bpmn-icon-gateway-xor',
      translate('Create Exclusive Gateway'),
    ),
    'create.parallel-gateway': createAction(
      'bpmn:ParallelGateway',
      'gateway',
      'bpmn-icon-gateway-parallel',
      translate('Create Parallel Gateway'),
    ),
    'create.inclusive-gateway': createAction(
      'bpmn:InclusiveGateway',
      'gateway',
      'bpmn-icon-gateway-or',
      translate('Create Inclusive Gateway'),
    ),
    // 'create.service-task': createAction(
    //   'bpmn:ServiceTask',
    //   'activity',
    //   'bpmn-icon-service-task',
    //   translate('Create Service Task'),
    // ),
    // 'create.data-object': createAction(
    //   'bpmn:DataObjectReference',
    //   'data-object',
    //   'bpmn-icon-data-object',
    //   translate('Create DataObjectReference'),
    // ),
    // 'create.data-store': createAction(
    //   'bpmn:DataStoreReference',
    //   'data-store',
    //   'bpmn-icon-data-store',
    //   translate('Create DataStoreReference'),
    // ),
    // 'create.subprocess-expanded': {
    //   group: 'activity',
    //   className: 'bpmn-icon-subprocess-expanded',
    //   title: translate('Create expanded SubProcess'),
    //   action: {
    //     dragstart: createSubprocess,
    //     click: createSubprocess,
    //   },
    // },
    // 'create.participant-expanded': {
    //   group: 'collaboration',
    //   className: 'bpmn-icon-participant',
    //   title: translate('Create Pool/Participant'),
    //   action: {
    //     dragstart: createParticipant,
    //     click: createParticipant,
    //   },
    // },
    // 'create.group': createAction(
    //   'bpmn:Group',
    //   'artifact',
    //   'bpmn-icon-group',
    //   translate('Create Group'),
    // ),
  });

  return actions;
};
