import { config } from '@crami/lowcode-vue-renderer';
import { exports } from '../../bootstrap';
import { Http } from '@crami/http';
import { HModal } from '@crami/ui';

interface ModelCommonParam {
  modelName: string;
  // dsName: string;
  // projectCode: string;
  // version: string;
}
interface ModelSaveParam {
  data: { [prop in string]: any }[];
}
interface ModelUpdateParam {
  id: string;
  data: { [prop in string]: any }[];
}
interface ModelDetailParam {
  id: string;
}
interface ModelDeleteParam {
  id: string;
}
interface ModelDeleteAllParam {
  ids: string[];
  deleteForeignAfterConfirmation?: boolean;
}
interface ModelBatchParam {
  ids: string[];
}

function commonParam() {
  const { projectCode, dsName, version, appId, appCode } = exports.router.currentRoute.value.meta;
  const userId = config.getGlobalDataSourceParams()?.userId;
  return {
    dsName,
    projectCode,
    appId,
    appCode,
    userId,
    version,
    lang: 'zhs',
  };
}

/** 单表 */

// 保存
export function saveByModel(param: ModelSaveParam & ModelCommonParam): Promise<any> {
  const _common = commonParam();
  return Http.post({ url: `/loki-server/domain/save`, params: Object.assign(_common, param) });
}

// 更新
export function updateByModel(param: ModelUpdateParam & ModelCommonParam): Promise<any> {
  const _common = commonParam();
  return Http.post({ url: `/loki-server/domain/update`, params: Object.assign(_common, param) });
}

// 删除
export function deleteByModel(param: ModelDeleteParam & ModelCommonParam): Promise<any> {
  const _common = commonParam();
  return Http.request({
    url: `/loki-server/domain/id`,
    method: 'DELETE',
    data: Object.assign(_common, param),
  });
}

// 批量删除
export async function deleteAllByModel(
  param: ModelDeleteAllParam & ModelCommonParam,
): Promise<any> {
  param.deleteForeignAfterConfirmation = false;

  const data = await getForeignKeyByModel(param.modelName);
  const hasTip = data?.some(foreignKey => {
    return foreignKey?.foreignKeyList.some(item => {
      // 当
      return item.deletingSourceType === 'deleteAfterConfirmation';
    });
  });
  if (hasTip) {
    param.deleteForeignAfterConfirmation = await new Promise(resolve => {
      HModal.confirm({
        title: '删除提示',
        content: '是否要删除从表数据?',
        okText: '删除',
        okType: 'danger',
        cancelText: '不删除',
        onOk() {
          resolve(true);
        },
        onCancel() {
          resolve(false);
        },
      });
    });
  }
  const _common = commonParam();
  return Http.request({
    url: `/loki-server/domain/ids`,
    method: 'DELETE',
    data: Object.assign(_common, param),
  });
}

// 单条详情
export function infoByModel(param: ModelDetailParam & ModelCommonParam): Promise<any> {
  const _common = commonParam();
  return Http.request({
    url: `/loki-server/domain/info`,
    method: 'GET',
    params: Object.assign(_common, param),
  });
}

// 批量详情
export function batchInfoByModel(param: ModelBatchParam & ModelCommonParam): Promise<any> {
  const _common = commonParam();
  return Http.request({
    url: `/loki-server/domain/batchInfo`,
    method: 'GET',
    params: Object.assign(_common, param),
  });
}

/** 多表 主外键 */

// 主外键保存
export function saveByModelFk(param): Promise<any> {
  const _common = commonParam();
  return Http.post({ url: '/loki-server/domain/saveFk', params: Object.assign(_common, param) });
}

// 主外键更新
export function updateByModelFk(param): Promise<any> {
  const _common = commonParam();
  return Http.post({ url: '/loki-server/domain/updateFk', params: Object.assign(_common, param) });
}

const modelInfoCache = new Map();
/**
 * 获取模型主外键信息
 * @param modelName
 * @returns
 */
export function getForeignKeyByModel(modelName): Promise<any> {
  const _cache = modelInfoCache.get(modelName);
  if (_cache) {
    return _cache;
  }
  const _common = commonParam();
  return Http.get({
    url: '/loki-server/model/foreignKey',
    params: {
      fileName: modelName,
      ..._common,
    },
  }).then(data => {
    modelInfoCache.set(modelName, data.data);
    return data.data;
  });
}
