import { defineComponent } from 'vue';
import { HModal } from '@crami/ui';
import VueRenderer from '@crami/lowcode-vue-renderer';
import { getSchema } from './getSchema';

const config = {
  create: {
    title: '新建对象',
  },
  update: {
    title: '编辑对象',
  },
  detail: {
    title: '浏览对象',
  },
};

const ModalPage = defineComponent({
  data() {
    return {
      title: '',
      visible: false,
      schema: '',
      components: [],
      modelName: '',
    };
  },
  props: ['params', 'routeMeta', 'userToken'],
  mounted() {},
  expose: ['open', 'close'],
  methods: {
    async init() {
      const { projectCode, appCode } = this.routeMeta;
      const { pageId, __pageType, modelName, title } = this.params;
      if (pageId) {
        const { schema, components } = await getSchema({
          pageId,
          projectCode,
          appCode,
        });
        this.schema = schema;
        this.components = components;
      } else if (__pageType && modelName) {
        this.schema = Object.assign({}, config[__pageType]?.schema);
      }
      this.title = title || config[__pageType]?.title;
    },
    open() {
      this.init();
      this.visible = true;
    },
    close() {
      this.params?.closeHandler();
      this.onCancel();
    },
    onCancel() {
      this.visible = false;
    },
  },
  render(props) {
    const { params, routeMeta, userToken } = props;
    return (
      <HModal
        footer={null}
        width="70%"
        visible={this.visible}
        title={this.title}
        destroyOnClose={true}
        onCancel={this.onCancel}
      >
        {this.schema && this.visible && (
          <VueRenderer
            scope={{
              ...params,
              routeMeta,
              userToken,
              closePage: this.close,
              cancelPage: this.onCancel,
            }}
            schema={this.schema}
            components={this.components}
          ></VueRenderer>
        )}
      </HModal>
    );
  },
});

export default ModalPage;
