import { reactive } from 'vue';
import { isJSFunction, isJSExpression } from '@alilc/lowcode-types';
import { cloneDeep, isUndefined } from 'lodash-es';
// 调用组件方法
export function callComponentMethod(
  this: any,
  params: { component: string; method: string; addImportParam: boolean; params: any },
  importParam: any,
) {
  const { component, method, addImportParam, params: _params } = params;
  // 处理单参数的情况
  if (isJSFunction(_params) || isJSExpression(_params)) {
    const value = this[_params.value.replace('__self.', '')];
    if (addImportParam) {
      this.$refs[component]?.[method](importParam, value);
    } else {
      this.$refs[component]?.[method](value);
    }
    return;
  }
  // 处理参数
  Object.keys(_params)?.map((key: string) => {
    if (isJSFunction(_params[key]) || isJSExpression(_params[key])) {
      const value = this[_params[key].value.replace('__self.', '')];
      _params[key] = value;
    }
  });
  if (addImportParam) {
    this.$refs[component]?.[method](importParam, _params);
  } else {
    this.$refs[component]?.[method](_params);
  }
}

// 从表单中获取filter信息，支持HBForm和ProForm
function getFilterByForm(formRef: string) {
  const instance = this.$refs[formRef];
  const formData = instance?.getData() || {};

  const schemas: any[] = instance.$props.schemas || instance.$attrs.schemas;

  return Object.keys(formData)
    .filter(key => !!formData[key])
    .map(key => {
      const formItem =
        schemas?.find(_schema => _schema.dataIndex === key || _schema.key === key) || {};
      return {
        field: key,
        value: formData[key],
        operation: formItem.operation || 'LIKE',
        andOrFlag: formItem.andOrFlag || 'AND',
      };
    });
}

// 表格搜索
export function tableSearch(this: any, params: { component: string; currentNodeId: string }) {
  const filter = getFilterByForm.bind(this)(params.currentNodeId);
  const _extraFilter = this.$refs[params.component].$attrs.extraFilter;
  let extraFilter = [];
  if (_extraFilter && _extraFilter.length > 0) {
    extraFilter = _extraFilter.filter(config => {
      return !isUndefined(config.value);
    });
  }
  this.dataSourceMap?.[`${params.component}__data`]?.load({
    filter: [...filter, ...extraFilter],
  });
}

// 主从表格搜索
export function doubleTableSearch(
  this: any,
  params: { mainTable: string; fromTable: string; currentNodeId: string },
) {
  const filter = getFilterByForm.bind(this)(params.currentNodeId);
  const extraFilter = this.$refs[params.mainTable].$attrs.extraFilter || [];

  this.dataSourceMap?.[`${params.mainTable}__data`]?.load({
    filter: [...filter, ...extraFilter],
  });
  this[`${params.fromTable}__data`] = { data: [], total: 0 };
  // this.$refs[params.fromTable].setData([]);
}

interface FromTableSearchParams {
  fromTable: string;
  parentField: {
    type: string;
    name: string;
    field: string;
  };
  childField: {
    type: string;
    name: string;
    field: string;
  };
}

// 子表搜索
export function fromTableSearch(this: any, params: FromTableSearchParams, event: any, record: any) {
  const { fromTable, parentField, childField } = params;
  this.dataSourceMap[`${fromTable}__data`].load({
    filter: [
      {
        field: childField.field,
        value: record[parentField.field],
        operation: '=',
        andOrFlag: 'AND',
      },
    ],
  });
}

// 搜索所有表格
export function tableSearchAll(this: any) {
  Object.values(this.$refs).map((ref: any) => {
    console.log(ref.type);
  });
}

export function arrParamsToObjParams(params: { name: string; value: any }[]) {
  const _params = [];
  params.map(param => {
    const { name, value } = param;
    _params.push({
      field: name,
      value: value,
      operation: '=',
      andOrFlag: 'AND',
    });
  });
  return _params;
}

/**
 * 树表格根节点获取
 * @param this
 * @param params
 */
export async function treeTableSearch(
  this: any,
  params: { component: string; currentNodeId: string },
) {
  const { component, currentNodeId } = params;
  const key = component || currentNodeId;
  const dataSourceId = `${key}__data`;
  const instance = this.$refs[key];
  const config = instance.$attrs.__treeTableConfig;

  this[dataSourceId] = reactive({
    total: 0,
    data: [],
  });

  // 获取根数据
  config.map(async child => {
    const modelName = child.config.modelName;
    const params = arrParamsToObjParams(child.config.params);

    const { total, data } = await this.dataSourceMap[dataSourceId].load(
      {
        filter: [...params],
        modelName,
      },
      { assignToScope: false },
    );

    data?.map(item => {
      item.children = [];
      item.__node_key__ = child.key;
    });
    const { total: prevTotal, data: prevData } = this[dataSourceId];
    this[dataSourceId] = {
      total: total + prevTotal,
      data: [...prevData, ...data],
    };
  });
}

// 根据属性数据和key value获取数据
export function getTreeNodeDataByKey(data: any[], key: string, value: any) {
  if (!data?.length) {
    return null;
  }
  let _cache = null;
  const fn = arr => {
    arr.some(item => {
      if (item[key] === value) {
        return (_cache = item);
      } else if (item?.children?.length) {
        fn(item.children);
      }
    });
  };
  fn(data);
  return _cache;
}

/**
 * 树表格展开
 * @param this
 * @param params
 * @param rest
 * @returns
 */
export async function treeTableExpand(
  this: any,
  params: { currentNodeId: string; finish?: boolean },
  ...rest
) {
  if (!rest[0]) {
    return false;
  }
  const currentRow = rest[1];
  const { currentNodeId, finish } = params;
  // 如果已经存在数据，并且不是强制刷新
  if (currentRow?.children?.length && !finish) {
    return false;
  }
  currentRow.children = [];

  // 获取信息
  const key = currentNodeId;
  const instance = this.$refs[key];
  const config = instance.$attrs.__treeTableConfig;
  const rowKey = instance.$props.rowKey;
  const dataSourceId = `${key}__data`;

  // 获取所在位置和节点
  const __parentNodeKey = currentRow.__node_key__;
  const __parentNode = getTreeNodeDataByKey(config, 'key', __parentNodeKey);

  // 获取子数据
  __parentNode?.children?.map(async child => {
    const modelName = child.config.modelName;
    const params = arrParamsToObjParams(child.config.params);
    // 组装查询参数
    const filter = [
      {
        field: child.config.searchField,
        value: currentRow[__parentNode.config.valueField],
        operation: '=',
        andOrFlag: 'AND',
      },
      ...params,
    ];
    const { data } = await this.dataSourceMap[dataSourceId].load(
      {
        filter,
        modelName,
      },
      { assignToScope: false },
    );
    // 配置数据信息
    data.map(item => {
      if (child?.children?.length) {
        item.children = [];
      }
      item.__node_key__ = child.key;
    });
    const currentData = getTreeNodeDataByKey(this[dataSourceId].data, rowKey, currentRow[rowKey]);
    currentData.children.push(...data);
    console.log(this[dataSourceId]);
  });
}

/**
 * 联动Tab组
 * @param this
 * @param params
 * @param eventParam
 */
export function selectLinkTabs(
  this: any,
  params: { component: string; currentNodeId: string },
  selectedKeys: Array<string>,
  node: any,
) {
  const __treeDatasourceConfig = this.$refs[params.currentNodeId].__treeDatasourceConfig;
  const getNodeByKey = (nodeKey: string) => {
    const getNodeDeep = (children: any[]): any => {
      let tempNode: Node | null = null;
      children.forEach((_node: any) => {
        if (_node.key === nodeKey) {
          tempNode = _node;
        } else {
          if (_node.children && _node.children.length > 0) {
            tempNode = getNodeDeep(_node.children);
          }
        }
      });
      return tempNode;
    };
    return getNodeDeep(__treeDatasourceConfig);
  };
  // 基于视图的数据源，value5取的就是节点上的key，自定义数据源则要自己塞上NodeKey，
  // 因为访问的是通用列表查询，后台不存在节点的概念
  const nodeKey = node.node.dataRef.value5 || node.node.dataRef.NodeKey;
  const nodeId = node.node.dataRef.value1 || node.node.dataRef.EID;
  const tempNode = getNodeByKey(nodeKey);
  const linkTabs = tempNode?.linkConfig?.PageLinkList;
  if (linkTabs) {
    const nodeData = node.node.dataRef._nodeData || {};
    this.$refs[params.component]?.selectLinkTabs(linkTabs, {
      ...tempNode,
      nodeKey: nodeId,
      ...nodeData,
    });
  }
}

/**
 * 懒加载子节点
 * @param params 参数
 * @param node 点击的节点
 * @returns
 */
export function proTreeLoadData(
  this: any,
  params: { dataSourceId: string; treeViewName: string },
  node: any,
) {
  if (node.isLeaf) {
    return Promise.resolve(true);
  }
  if (node.dataRef.children && node.dataRef.children.length > 0) {
    return Promise.resolve(true);
  }
  const { value1, value2, value3, value4, value5, value6 } = node.dataRef;
  const _params = {
    treeViewName: params.treeViewName,
    loadAllStructure: false,
    value1: value1,
    value2: value2,
    value3: value3,
    value4: value4,
    value5: value5,
    value6: value6,
  };
  return this.dataSourceMap[params.dataSourceId].load(_params).then(childData => {
    if (childData && childData.length > 0) {
      node.dataRef.children = [...childData];
    } else {
      node.dataRef.isLeaf = true;
    }
    return true;
  });
}

function getListByNodes(nodesConfig: any[], treeDataSource) {
  const promiseArr = nodesConfig
    .map(node => {
      if (node?.nodeConfig?.DataSource?.name) {
        const { name, field } = node.nodeConfig.DataSource;
        return treeDataSource.load({ modelName: name }).then(res => {
          if (res && res.total > 0) {
            const data = res.data;
            return data.map(item => {
              return {
                key: `${node.key}_${item.EID}`,
                title: item[field],
                isLeaf: node.isLeaf,
                NodeKey: node.key,
                DataSource: cloneDeep(node.DataSource),
                childNodesConfig: cloneDeep(node.children),
                _nodeData: item,
              };
            });
          }
          return [];
        });
      } else {
        // 固定节点
        return Promise.resolve([
          {
            key: `${node.key}_${Date.now()}`,
            title: node.nodeConfig.NodeName,
            isLeaf: node.isLeaf,
            NodeKey: node.key,
            DataSource: cloneDeep(node.DataSource),
            childNodesConfig: cloneDeep(node.children),
          },
        ]);
      }
    })
    .filter(item => item);
  if (promiseArr.length > 0) {
    return Promise.all(promiseArr)
      .then(res => {
        if (res && res.length > 0) {
          let resData = [];
          res.forEach(item => {
            if (item && item.length > 0) {
              resData = resData.concat(item);
            }
          });
          return resData;
        }
      })
      .catch(err => {
        console.error(err);
      });
  }
  return Promise.resolve([]);
}

/**
 * 加载根节点数据
 * @param this
 * @param params
 */
export function loadTreeData(this: any, params: { currentNodeId: string }) {
  const nodeId = params.currentNodeId;
  const instance = this.$refs[nodeId];
  const dataSourceId = `treeDataSource_${nodeId}`;
  // 查找根节点数据源
  const nodes = instance.__treeDatasourceConfig || [];
  getListByNodes(nodes, this.dataSourceMap[dataSourceId]).then(resData => {
    if ((resData as any[]).length > 0) {
      instance.$.props.treeData = cloneDeep(resData);
    }
  });
}

/**
 * 加载子节点数据
 * @param this
 * @param params
 */
export function loadChildTreeData(this: any, params: { currentNodeId: string }, node: any) {
  if (node.isLeaf) {
    return Promise.resolve(true);
  }
  if (node.dataRef.HasLoad) {
    return Promise.resolve(true);
  }
  node.dataRef.children = [];
  const nodeId = params.currentNodeId;
  const dataSourceId = `treeDataSource_${nodeId}`;
  // 查找根节点数据源
  const nodes = node.dataRef.childNodesConfig;
  return getListByNodes(nodes, this.dataSourceMap[dataSourceId]).then(resData => {
    node.dataRef.HasLoad = true;
    const resArr = resData as any[];
    if (resArr.length > 0) {
      node.dataRef.children = cloneDeep(resArr);
    } else {
      node.dataRef.isLeaf = true;
    }
    return true;
  });
}
