import { getTreeNodeDataByKey } from '../../utils';
import { message } from '@crami/ui';
import {
  deleteAllByModel,
  infoByModel,
  saveByModel,
  updateByModel,
  saveByModelFk,
  updateByModelFk,
} from '../../api/commonApi/model';
import { openModal } from './modal';
import { treeTableExpand, treeTableSearch } from './method';

// 新建表单
export function createObject(
  this: any,
  params: { modelName: string; pageId: string; component: string },
) {
  const { pageId, component } = params;
  openModal.bind(this)({
    pageId,
    __pageType: 'create',
    closeHandler: () => {
      // 树表格
      if (this.$refs[component].$attrs.__treeTableConfig) {
        treeTableSearch.bind(this)({ component });
      } else {
        // 普通表格
        this.reloadDataSource(`${component}__data`);
      }
    },
  });
}

// 新建子数据表单
export function addChildObject(
  this: any,
  params: { modelName: string; pageId: string; component: string },
) {
  const { modelName, pageId, component } = params;
  const instance = this.$refs[component];
  const rows = instance.getSelectRows();
  if (!rows?.length) {
    message.warn('请选择一条数据');
    return false;
  }
  const currentRow = rows[0];
  const config = instance.$attrs.__treeTableConfig;
  const __parentNodeKey = currentRow.__node_key__;
  const __parentNode = getTreeNodeDataByKey(config, 'key', __parentNodeKey);

  const currentNode = __parentNode?.children?.find(child => child?.config?.modelName === modelName);
  if (!currentNode) {
    message.warn('此数据下不存在这种子数据');
    return false;
  }

  const __extendData = {
    [currentNode.config.searchField]: currentRow[__parentNode.config.valueField],
  };

  openModal.bind(this)({
    pageId,
    __extendData,
    __pageType: 'create',
    closeHandler: () => {
      currentRow.children = [];
      treeTableExpand.bind(this)({ currentNodeId: component, finish: true }, true, currentRow);
    },
  });
}

// 编辑表单
export async function editObject(
  this: any,
  params: { modelName: string; pageId: string; component: string },
) {
  const { modelName, pageId } = params;
  const instance = this.$refs[params.component];
  const primaryCode = instance.$props.rowKey;
  const rows = instance.getSelectRows();
  if (!rows?.length) {
    return false;
  }
  const res = await infoByModel({
    id: rows[0][primaryCode],
    modelName,
  });
  openModal.bind(this)({
    pageId,
    __pageType: 'update',
    formInitialData: res.data,
    closeHandler: () => {
      this.reloadDataSource(`${params.component}__data`);
    },
  });
}

// 浏览表单
export async function detailObject(
  this: any,
  params: { modelName: string; pageId: string; component: string },
) {
  const { modelName, pageId } = params;
  const instance = this.$refs[params.component];
  const primaryCode = instance.$props.rowKey;
  const rows = instance.getSelectRows();
  if (!rows?.length) {
    return false;
  }
  const res = await infoByModel({
    id: rows[0]?.[primaryCode],
    modelName,
  });
  openModal.bind(this)({
    pageId,
    __pageType: 'detail',
    formInitialData: res.data,
  });
}

// 保存表单，分为单表、多表
export function saveChanges(this: any, params: { currentNodeId: string }) {
  const { currentNodeId } = params;
  const instance = this.$refs[currentNodeId];
  const compName = instance.$.type.name;

  // 单表类型的
  if (['HBForm', 'ProForm'].includes(compName)) {
    saveChangesBySingle.bind(this)(instance);
  }
  // 主外键类型的
  if (['HButton'].includes(compName)) {
    saveChangesByMultip.bind(this)();
  }
}

// 单表保存
function saveChangesBySingle(this: any, instance) {
  const formData: any = instance.getData();
  const primaryCode = instance.$attrs.primaryCode;
  const modelName = this.modelName || instance.$attrs.__modelName || instance.modelName || '';
  if (!modelName || !formData) {
    return;
  }
  // 混入外部传入的数据
  if (this.__extendData) {
    Object.assign(formData, this.__extendData);
  }
  if (this.__pageType === 'create' || !this.__pageType) {
    instance.submitButtonLoading();
    delete formData[primaryCode];
    saveByModel({
      data: formData,
      modelName,
    })
      .then((data: any) => {
        this?.closePage(data);
      })
      .finally(() => {
        instance.submitButtonLoading();
      });
    return false;
  }
  if (this.__pageType === 'update') {
    instance.submitButtonLoading();
    delete formData['CREATETIME'];
    updateByModel({
      id: formData[primaryCode],
      data: formData,
      modelName,
    })
      .then((data: any) => {
        this?.closePage(data);
      })
      .finally(() => {
        instance.submitButtonLoading();
      });
    return false;
  }
}

// 多表保存
function saveChangesByMultip(this: any) {
  const instanceKeys = Object.keys(this.$refs);
  const forms = instanceKeys
    .filter(key => ['HBForm', 'ProForm'].includes(this.$refs[key].$.type.name))
    .map(key => this.$refs[key]);
  const tables = instanceKeys
    .filter(key => this.$refs[key].$.type.name === 'HBProTable')
    .map(key => this.$refs[key]);

  const modelName = forms[0].$attrs.__modelName;
  const primaryCode = forms[0].$attrs.primaryCode;

  const formData: any = forms[0].getData();
  delete formData['CREATETIME'];

  const pageData = {};
  Object.keys(formData).map(key => {
    pageData[`${modelName}_${key}`] = formData[key];
  });

  if (this.__pageType === 'create' || !this.__pageType) {
    tables.map(_table => {
      const keys = _table.$props.columns
        .filter(column => column.dataIndex !== '__pro-table-custom-operation__')
        .map(column => column.key || column.dataIndex);
      const tableModelName = _table.$attrs.__modelName;

      pageData[tableModelName] = _table
        .getDisplayAllData()
        .map(setModelNameToTableDataKey(keys, tableModelName));
    });
    delete pageData[`${modelName}_${primaryCode}`];
    saveByModelFk({
      data: pageData,
      modelName,
    }).then((data: any) => {
      this?.closePage(data);
    });
  }
  if (this.__pageType === 'update') {
    tables.map(_table => {
      const keys: string[] = _table.$props.columns
        .filter(column => column.dataIndex !== '__pro-table-custom-operation__')
        .map(column => column.key || column.dataIndex);
      const tableModelName = _table.$attrs.__modelName;
      const primaryCode = _table.$props.rowKey;

      pageData[`${tableModelName}-insert`] = _table
        .getAddRows()
        .map(setModelNameToTableDataKey(keys, tableModelName));
      const updateKeys: string[] = [...keys];
      if (!keys.includes(primaryCode)) {
        updateKeys.push(primaryCode);
      }
      pageData[`${tableModelName}-update`] = _table
        .getUpdateRows()
        .map(setModelNameToTableDataKey(updateKeys, tableModelName));
      pageData[`${tableModelName}-delete`] = _table.getDelRows().map(row => row[primaryCode]);
    });
    updateByModelFk({
      id: pageData[`${modelName}_${primaryCode}`],
      data: pageData,
      modelName,
    }).then((data: any) => {
      this?.closePage(data);
    });
  }
}

// 取消保存
export function cancelChanges(this: any) {
  this?.cancelPage();
}

// 删除记录
export function deleteRecord(this: any, params: { component: string }) {
  const instance = this.$refs[params.component];
  const rows = instance.getSelectRows();
  const primaryCode = instance.$props.rowKey;
  if (!rows?.length) {
    return;
  }
  const modelName = instance.$attrs.__modelName || instance.modelName || this.modelName || '';
  deleteAllByModel({
    ids: rows.map(row => row[primaryCode]),
    modelName,
  }).then(() => {
    instance.removeSelectedRows();
  });
}

// 工具函数——————————————————————————————————

// 设置模型名称到表格数据上
function setModelNameToTableDataKey(keys, modelName) {
  return item => {
    const _rowData = {};
    keys.map(key => {
      _rowData[`${modelName}_${key}`] = item[key];
    });
    return _rowData;
  };
}
