import { getSchemaById as getSchemaFromServer } from '../../api/page';
import pkgJson from './packages.json';
import _projectSchema from './projectSchema.json';
import { buildComponents } from '@crami/lowcode-utils';
import { isNull, isUndefined } from 'lodash-es';
import { evaluate } from '@studio/formula';

export const getSchemaById = async (params: {
  pageId: string;
  projectCode: string;
  appCode: string;
}) => {
  const packages = pkgJson;
  let projectSchema: any = await getSchemaFromServer(params);
  if (!projectSchema.data) {
    projectSchema = _projectSchema;
  } else {
    projectSchema = projectSchema.data;
  }
  const { componentsMap: componentsMapArray, componentsTree } = projectSchema as any;
  const componentsMap: any = {};
  componentsMapArray.forEach((component: any) => {
    componentsMap[component.componentName] = component;
  });

  const libraryMap = {};
  packages.forEach(({ package: _package, library }) => {
    libraryMap[_package] = library;
  });
  const components = buildComponents(libraryMap, componentsMap);
  return { schema: componentsTree[0], components };
};
export interface ParamItem {
  key: string;
  label: string;
  valueType: string;
  required?: boolean;
}

export function normalizePageParams(
  pageParams: { [key: string]: any },
  pageParamsConfig: Array<ParamItem>,
  formularData: any,
) {
  let newParams = {};
  if (pageParamsConfig && pageParamsConfig.length > 0) {
    newParams = pageParamsConfig.reduce((params: { [key: string]: any }, curItem: ParamItem) => {
      const { valueType, key } = curItem;
      const value = pageParams[key];
      if (isUndefined(value)) {
        return params;
      }
      if (valueType === 'object' || valueType === 'array') {
        try {
          params[key] = JSON.parse(value);
        } catch (error) {
          console.warn(`json解析异常，数据 \n ${value}`);
        }
      } else if (valueType === 'formular') {
        // 执行公式
        if (!isNull(value)) {
          const result = evalFormula(value, formularData);
          if (!isUndefined(result)) {
            params[key] = result;
          }
        }
      } else {
        params[key] = value;
      }
      return params;
    }, {});
  }
  return newParams;
}

/**
 * 执行公式
 * @param expression 公式
 * @param data 变量值对象
 * @returns 执行结果
 */
export function evalFormula(expression: string, data?: object) {
  const curData = data || {};
  let result;
  try {
    result = evaluate(expression, curData, {
      evalMode: true, // evalMode 为 true 时，不用 ${} 包裹也可以执行，
      allowFilter: false,
    });
  } catch (e) {
    console.warn(
      '[evalFormula]表达式执行异常，当前表达式: ',
      expression,
      '，当前上下文数据: ',
      data,
    );
  }
  return result;
}
