import { defineComponent, onMounted, reactive, toRefs, h, createVNode, render, inject } from 'vue';
import VueRenderer from '@crami/lowcode-vue-renderer';
import { Spin } from '@crami/ui';
import './preview.css';
import ModalPage from '@/components/modal-page';
import { useRoute } from 'vue-router';
import { getSchemaById, normalizePageParams } from './helper';
import { previewProps } from './props';

export default defineComponent({
  name: 'EditorPreviewOnly',
  props: previewProps,
  mounted() {},
  setup(props) {
    const data = reactive<any>({});
    const { pageId } = toRefs(props);
    const projectCode = inject('projectCode', { value: '' });
    const appCode = inject('appCode', { value: '' });
    onMounted(async () => {
      if (pageId.value) {
        Object.assign(data, await getSchemaById(pageId.value, projectCode.value, appCode.value));
      }
    });
    const route = useRoute();
    const token: any = JSON.parse(localStorage.getItem('token') || '');
    const userToken = `Bearer ${token?.token || ''}`;

    return () => {
      const { schema, components } = data;
      if (!schema || !components) {
        return h(Spin);
      }

      async function openModal(params: any) {
        const vnode = createVNode(ModalPage, {
          params,
          components,
          routeMeta: route.meta,
          userToken,
        });
        render(vnode, document.body);
        vnode.component.ctx.open();
      }
      const pageParams = normalizePageParams(props.pageParams, schema.pageParamsConfig);
      return h('div', { class: 'lowcode-plugin-sample-preview' }, [
        h(VueRenderer, {
          scope: {
            openModal,
            routeMeta: route.meta,
            userToken,
            pageParams: pageParams,
          },
          class: 'lowcode-plugin-sample-preview-content',
          schema,
          components,
        }),
      ]);
    };
  },
});
